const {
  JuphoonWebRTCGuest,
  StompSocketAuthType,
  ConferenceEventType,
  SimpleRoleCompose,
  SimpleSVCResolution,
  StorageManagerInstance,
  WebMediaDevicesInstance,
  IRTCCoreEventType,
  GuestConferenceTag,
  ConferenceSecurityType,
  ConferenceMediaEncodeType,
  GuestEventType,
  PictureSize,
  SocketEventType
} = JRTCSDK;

const sdkVersionInfo = JuphoonWebRTCGuest.getVersion();
document.querySelector('.version-info').textContent = sdkVersionInfo.version + '-' + sdkVersionInfo.gitHashShort + '-' + sdkVersionInfo.buildTime;

const app = Vue.createApp({
  data() {
    return {
      StompSocketAuthType: StompSocketAuthType,
      GuestConferenceTag: GuestConferenceTag,
      ConferenceSecurityType: ConferenceSecurityType,
      ConferenceMediaEncodeType: ConferenceMediaEncodeType,
      PictureSize: PictureSize,
      client: {},
      environmentSettings: {},
      joinSettings: {},
      callAgentSettings: {},
      callSettings: {},
      WebMediaDevicesInstance: {}
    };
  },
  created() {
    WebMediaDevicesInstance.refresh()
      .then(() => this.WebMediaDevicesInstance = WebMediaDevicesInstance);

    this.reset();
    const settingFormData = StorageManagerInstance.get('settingFormData_webrtc-guest-sdk');

    if (settingFormData) {
      Object.entries(settingFormData)
        .forEach((entity) => this[entity[0]] = entity[1]);
    }

    window.onbeforeunload = (ev) => {
      const {environmentSettings, joinSettings, callSettings, callAgentSettings} = this;
      /**
       * 重置不需要保存的参数
       */
      environmentSettings.showReason = false;
      environmentSettings.environmentConfig.token = undefined;
      joinSettings.showReason = false;
      callSettings.statsInfo = '';

      StorageManagerInstance.set('settingFormData_webrtc-guest-sdk', {
        environmentSettings,
        joinSettings,
        callSettings,
        callAgentSettings
      });

      ev.preventDefault();
      ev.returnValue = '';
      return '';
    };
  },
  unmounted() {
  },
  methods: {
    setConfig() {
      JuphoonWebRTCGuest.setConfig(this.environmentSettings.environmentConfig);
    },
    getToken() {
      this.environmentSettings.environmentConfig.authType = StompSocketAuthType.REDIS;

      this.setConfig();
      JuphoonWebRTCGuest.getToken()
        .then((token) => this.environmentSettings.environmentConfig.token = token)
        .catch((reason) => this.handleEnvironmentSettingsError(reason));
    },
    login() {
      new Promise((resolve, reject) => {
        this.client = JuphoonWebRTCGuest.createGuestClient(this.environmentSettings.environmentConfig.appKey, this.environmentSettings.environmentConfig.token);

        this.client.addEventListeners(
          [ConferenceEventType.CONFERENCE_PARTICIPANT_JOIN, ConferenceEventType.CONFERENCE_PARTICIPANT_UPDATE, ConferenceEventType.CONFERENCE_PARTICIPANT_LEFT, ConferenceEventType.CLIENT_STATE_CHANGE],
          (ev) => {
            this.$forceUpdate();
          }
        );

        this.client.addEventListener(GuestEventType.CALL_QUEUE_COUNT, (ev) => {
          const {time, count} = ev.message.data;

          this.callAgentSettings.queueInfo.time = time;
          this.callAgentSettings.queueInfo.count = count;
        });

        this.client.addEventListener(ConferenceEventType.CONFERENCE_PROPERTY_CHANGE, (ev) => {
          const {screenShare} = ev.message.data.changeParam,
                participant = ev.message.data.participant;

          if (screenShare) {
            if (this.client.screenUserId) {
              this.client.startScreenVideo(this.client.screenRenderId)
                .then((stream) => {
                  this.screenStream = stream;
                  this.screenStream.start('screen-stream');
                });
            } else {
              this.screenStream.stop();
            }
          }
        });

        this.client.socket.addEventListener(SocketEventType.ERROR, (ev) => {
          console.log(ev);
        });

        this.client.login()
          .then(resolve)
          .catch(reject);
      })
        .catch((reason) => this.handleEnvironmentSettingsError(reason));

    },
    handleEnvironmentSettingsError(reason) {
      console.log(reason.message, reason.stack);
      this.environmentSettings.reason = reason.message;
      this.environmentSettings.showReason = true;
    },
    handleCallSettingsError(reason) {
      console.log(reason.message, reason.stack);
      this.callSettings.reason = reason.message;
      this.callSettings.showReason = true;
    },
    join() {
      this.client.join(this.joinSettings.conferenceNumber, this.joinSettings.joinParam)
        .then(() => this.client.startCameraVideo())
        .then((stream) => stream.start('local-stream', document))
        .then(() => this.client.startVideo())
        .then((stream) => stream.start('remote-stream', document))
        .then(() => this.handleShowStats(true))
        .catch((reason) => {
          this.joinSettings.reason = reason.message;
          this.joinSettings.showReason = true;
        });
    },
    logout() {
      this.client.logout()
        .then(() => this.handleShowStats(false));
    },
    leave() {
      this.client.leave()
        .then(() => this.handleShowStats(false));
    },
    reset() {
      this.environmentSettings = {
        environmentConfig: {
          address: 'https://videobu.juphoon.cloud:10047',
          appKey: '05c97ce0d0a84d2592334093',
          token: '',
          authType: StompSocketAuthType.REDIS
        },
        sendOnlineMessageParam: {
          toUserId: '',
          content: '',
        },
        roomId: undefined,
        showReason: false,
        reason: undefined
      };
      this.joinSettings = {
        conferenceNumber: '',
        joinParam: {
          displayName: '',
          conferencePassword: '123456',
          enableRecord: false,
          capacity: 6,
          role: SimpleRoleCompose.NORNAL,
          svcResolution: SimpleSVCResolution.HD,
          wholeRatio: 0.5625,
          isZanTong: false,
        },
        showReason: false,
        reason: undefined
      };
      this.callAgentSettings = {
        telNumber: '10086',
        agentUserId: '',
        callParam: {
          displayName: '',
          extraParam: '',
          svcResolution: SimpleSVCResolution.SD,
          wholeRatio: 0.5625,
          token: '',
        },
        queueInfo: {
          time: 0,
          count: 0
        },
        showReason: false,
        reason: ''
      };
      this.callSettings = {
        cameraConstraints: {},
        sendMessageParam: {
          type: '',
          toUserId: '',
          content: '',
        },
        requestVideoParam: {
          participant: {},
          pictureSize: PictureSize.MIN
        },
        ratio: '',
        showStats: false,
        statsInfo: '',
        showReason: false,
        reason: undefined
      };
    },
    handleCameraChanged() {
      this.client.switchCamera(this.callSettings.cameraConstraints);
    },
    triggerUploadVideoStream() {
      this.client.localStream.videoState ? this.client.disableUploadVideoStream() : this.client.enableUploadVideoStream();
    },
    triggerUploadAudioStream() {
      this.client.localStream.audioState ? this.client.disableUploadAudioStream() : this.client.enableUploadAudioStream();
    },
    sendOnlineMessage() {
      const {toUserId, content} = this.environmentSettings.sendOnlineMessageParam;
      this.client.sendOnlineMessage(toUserId, content)
        .catch((reason => this.handleEnvironmentSettingsError(reason)));
    },
    sendMessage() {
      const {type, content, toUserId} = this.callSettings.sendMessageParam;

      this.client.sendMessage(type, content, toUserId)
        .catch((reason => this.handleEnvironmentSettingsError(reason)));
    },
    query() {
      this.client.query(this.environmentSettings.roomId)
        .then((data) => this.handleEnvironmentSettingsError({message: JSON.stringify(data.data, undefined, 2)}))
        .catch((reason) => this.handleEnvironmentSettingsError(reason));
    },
    handleShowStats(showStats) {
      this.callSettings.showStats = showStats;

      this.client.core.getStats(this.callSettings.showStats ? 1000 : 0);
      this.client.core.addEventListener(IRTCCoreEventType.STATS, (ev) => {
        this.callSettings.statsInfo = JSON.stringify(ev.message.data.stats, undefined, 2);
      });
    },
    handleCall(callType) {
      let telNumberOrAgentUserId;

      switch (callType) {
        case 'oneToOneCall':
          telNumberOrAgentUserId = this.callAgentSettings.agentUserId;
          break;
        case 'call':
          telNumberOrAgentUserId = this.callAgentSettings.telNumber;
          break;
      }
      this.client[callType](telNumberOrAgentUserId, this.callAgentSettings.callParam)
        .then(() => this.client.startCameraVideo())
        .then((stream) => stream.start('local-stream', document))
        .then(() => this.client.startVideo())
        .then((stream) => stream.start('remote-stream', document))
        .then(() => this.handleShowStats(true))
        .catch((reason) => {
          this.callAgentSettings.reason = reason.message;
          this.callAgentSettings.showReason = true;
        });
    },
    term() {
      this.client.term()
        .then(() => this.handleShowStats(false));
    },
    requestVideo() {
      this.client.requestVideo(this.callSettings.requestVideoParam.participant, this.callSettings.requestVideoParam.pictureSize)
        .catch((reason => this.handleCallSettingsError(reason)));
    },
    setRatio() {
      this.client.setRatio(this.callSettings.ratio)
        .catch((reason => this.handleCallSettingsError(reason)));
    },
    urgent() {
      this.client.requestUrgent()
        .catch((reason => this.handleCallSettingsError(reason)));
    }
  }
})
  .mount('#app');
